require("kara3d.transform")

-- ********** DEFINITION AND GENERIC FUNCTIONS **********

camera = {class = "camera"}
camera.__index = camera

-- Creates a camera at transform t with aspect ratio a, horizontal fov f,
-- near plane distance nd and far plane distance fd
function camera.new(t, f, nd, fd)
  local c = {transform = t, fov = f or 80 / 180 * math.pi,
    near = nd or 1, far = fd or 1000000000}
  setmetatable(c, camera)
  return c
end

-- Clones this camera
function camera:clone()
  return camera.new(self.transform, self.fov, self.near, self.far)
end

-- ********** INSTANCE FUNCTIONS **********

-- Returns this camera's projection * view matrix
function camera:matrix()
  local s = 1 / math.tan(self.fov / 2)
  local n, f = self.near, self.far
  local xres, yres, _, _ = aegisub.video_size()
  local m = matrix.new(4, 4,
    {s, 0,             0,             0,
     0, s,             0,             0,
     0, 0, (n+f) / (n-f), 2*n*f / (n-f),
     0, 0,      1 / yres,             0})
  return m * self.transform:inv_matrix(true)
end
